@@ -9,6 +9,7 @@ use XML::Simple;
 use Digest::MD5 qw(md5_hex);
 use Encode;
 use Image::Magick;
+use Carp;
 
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
 
@@ -17,7 +18,7 @@ require Exporter;
 
 our @ISA = qw(Exporter AutoLoader);
 our @EXPORT = qw();
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 
 =head1 NAME
@@ -48,18 +49,63 @@ Create new object instance.
 =cut
 sub new {
 	my $class = shift;
+  my %passed_parms = @_;
 	my $self  = {};
 	$self->{API} = 'http://www.rajce.idnes.cz/liveAPI/index.php';
 	$self->{XML} = '<?xml version="1.0" encoding="utf-8"?>';
+	$self->{DEBUG} = $passed_parms{'debug'};
 	$self->{BOT} = WWW::Mechanize->new(autocheck => 1, agent => 'github.com/petrkle/rajce');
 	$self->{BOT}->add_header('Accept-Encoding'=>'text/html');
 	$self->{BOT}->add_header('Accept-Charset'=>'utf-8');
 	$self->{BOT}->add_header('Accept-Language'=>'cs');
 	$self->{BOT}->cookie_jar(HTTP::Cookies->new());
+
+	$self->{ERRORS} = {
+	'1' => 'Unknown error.',
+	'2' => 'Invalid command.',
+	'3' => 'Invalid login or password.',
+	'4' => 'Bad login token.',
+	'5' => 'Unknown or repeating column {colName}.',
+	'6' => 'Not correct albumID.',
+	'7' => 'Album not exist or logged user is not owner.',
+	'8' => 'Bad album token.',
+	'9' => 'Albumn cant have empty title.',
+	'10' => 'Failed to create new album. (hard to say why ... probably an error on the server side).',
+	'11' => 'Album not exist.',
+	'12' => 'Non existing application.',
+	'13' => 'Wrong application key.',
+	'14' => 'File is not attached.',
+	'15' => 'Already there is a newer version {version}.',
+	'16' => 'Error when saving a file.',
+	'17' => 'Illegal file extension {extension}.',
+	'18' => 'Wrong version number of client.',
+	'19' => 'No such object (target).',
+	'20' => 'Missing name to protect the album.',
+	'21' => 'Missing password to protect the album.',
+	'22' => 'Error communication - arrived an empty file.',
+	'23' => 'Some blocks of the video are missing.',
+	'24' => 'User does not exist.',
+	'25' => 'There is the correct userID or albumID.',
+	'26' => 'Album not exist or is not belog to this user or isnt public.',
+	'27' => 'Invalid clientVideoID.',
+	'28' => 'Upload with a given number does not exist.',
+
+	};
+
 	bless($self, $class);
 	return $self;
 }
 
+=item * $rajce->_debug($mesage);
+Show debugging message.
+=cut
+sub _debug{
+	my ($self,$message) = @_;
+	if($self->{DEBUG}){
+		print encode("utf8",$message)."\n";
+	}
+}
+
 =item * $rajce->login($mail,$password);
 Login to API.
 =cut
@@ -77,14 +123,22 @@ sub login {
 			},
 		}
 	};
-	my $response = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($login, KeepRoot => 1, XMLDecl => $self->{XML})});
-	my $login_resp = XMLin($response->content());
-	$self->{sessionToken}=$login_resp->{sessionToken};
-	$self->{maxWidth}=$login_resp->{maxWidth};
-	$self->{maxHeight}=$login_resp->{maxHeight};
-	$self->{nick}=$login_resp->{nick};
-	return $login_resp;
+
+	my $xml = XMLout($login, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $sign = $self->{BOT}->post($self->{API}, {'data' => $xml});
+	my $response = XMLin($sign->content());
+	$self->_debug($sign->content());
+
+	if($response->{errorCode}){
+		confess($self->{ERRORS}{$response->{errorCode}});
+	}
+
+	$self->{sessionToken}=$response->{sessionToken};
+	$self->{maxWidth}=$response->{maxWidth};
+	$self->{maxHeight}=$response->{maxHeight};
+	$self->{nick}=$response->{nick};
+	return $response;
 }
 
 =item * $rajce->list($userid);
@@ -102,8 +156,10 @@ sub list {
 		}
 	};
 
-	my $albums = $self->{BOT}->post($self->{API},
-	 {'data' => XMLout($listalbums, KeepRoot => 1, XMLDecl => $self->{XML})});
+	my $xml = XMLout($listalbums, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $albums = $self->{BOT}->post($self->{API}, {'data' => $xml});
+	$self->_debug($albums->content());
 
 return XMLin($albums->content());
 }
@@ -133,8 +189,10 @@ sub photo_list {
 		}
 	};
 
-	my $photos = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($photolist, KeepRoot => 1, XMLDecl => $self->{XML})});
+	my $xml = XMLout($photolist, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $photos = $self->{BOT}->post($self->{API}, {'data' => $xml});
+	$self->_debug($photos->content());
 
 return XMLin($photos->content());
 }
@@ -163,8 +221,10 @@ sub search_users {
 		}
 	};
 
-	my $result = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($users, KeepRoot => 1, XMLDecl => $self->{XML})});
+	my $xml = XMLout($users, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $result = $self->{BOT}->post($self->{API},	{'data' => $xml});
+	$self->_debug($result->content());
 
 return XMLin($result->content());
 }
@@ -185,9 +245,11 @@ sub get_url {
 		}
 	};
 
-	my $result = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($geturl, KeepRoot => 1, XMLDecl => $self->{XML})});
+	my $xml = XMLout($geturl, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $result = $self->{BOT}->post($self->{API},	{'data' => $xml});
 	my $response = XMLin($result->content());
+	$self->_debug($response->content());
 
 return $response->{url};
 }
@@ -219,11 +281,12 @@ sub search_albums {
 		}
 	};
 
-	print Dumper(XMLout($albums, KeepRoot => 1, XMLDecl => $self->{XML}));
-	my $result = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($albums, KeepRoot => 1, XMLDecl => $self->{XML})});
+	my $xml = XMLout($albums, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $response = $self->{BOT}->post($self->{API},	{'data' => $xml});
+	$self->_debug($response->content());
 
-return XMLin($result->content());
+return XMLin($response->content());
 }
 
 
@@ -233,16 +296,23 @@ Get URL where is form for creating new account on rajce.net.
 sub reg_url {
 	my ($self) = @_;
 
-	my $reg = {'request'=>{
+	my $request = {'request'=>{
 			'command'=>['getRegisterUrl']
 		}
 	};
 
-	my $regurl = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($reg, KeepRoot => 1, XMLDecl => $self->{XML})});
-	my $reg_form = XMLin($regurl->content());
+	my $xml = XMLout($request, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $regurl = $self->{BOT}->post($self->{API},	{'data' => $xml});
+	$self->_debug($regurl->content());
 
-return $reg_form->{url};
+	my $response = XMLin($regurl->content());
+
+	if($response->{errorCode}){
+		confess($self->{ERRORS}{$response->{errorCode}});
+	}
+
+return $response->{url};
 }
 
 =item * $rajce->recover_url();
@@ -251,16 +321,23 @@ Get URL where is form for recover forget password.
 sub recover_url {
 	my ($self) = @_;
 
-	my $pass = {'request'=>{
+	my $request = {'request'=>{
 			'command'=>['getRecoverPasswordUrl']
 		}
 	};
 
-	my $url = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($pass, KeepRoot => 1, XMLDecl => $self->{XML})});
-	my $pass_form = XMLin($url->content());
+	my $xml = XMLout($request, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $url = $self->{BOT}->post($self->{API}, {'data' => $xml});
+	$self->_debug($url->content());
 
-return $pass_form->{url};
+	my $response = XMLin($url->content());
+
+	if($response->{errorCode}){
+		confess($self->{ERRORS}{$response->{errorCode}});
+	}
+
+return $response->{url};
 }
 
 =item * $rajce->create_album($title,$desc);
@@ -280,10 +357,17 @@ sub create_album {
 		}
 	};
 
-	my $album = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($create, KeepRoot => 1, XMLDecl => $self->{XML})});
+	my $xml = XMLout($create, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $album = $self->{BOT}->post($self->{API}, {'data' => $xml});
+	$self->_debug($album->content());
 
-return XMLin($album->content());
+	my $response = XMLin($album->content());
+	if($response->{errorCode}){
+		confess($self->{ERRORS}{$response->{errorCode}});
+	}
+
+return $response;
 }
 
 =item * $rajce->_open_album($album);
@@ -292,7 +376,7 @@ Open album for adding pictures.
 sub _open_album {
 	my ($self,$album) = @_;
 
-	my $create = {'request'=>{
+	my $request = {'request'=>{
 			'command'=>['openAlbum'],
 			'parameters'=>{
 				'token'=>[$self->{sessionToken}],
@@ -301,10 +385,18 @@ sub _open_album {
 		}
 	};
 
-	my $open = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($create, KeepRoot => 1, XMLDecl => $self->{XML})});
+	my $xml = XMLout($request, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $open = $self->{BOT}->post($self->{API},	{'data' => $xml});
+	$self->_debug($open->content());
 
-return XMLin($open->content());
+	my $response = XMLin($open->content());
+
+	if($response->{errorCode}){
+		confess($self->{ERRORS}{$response->{errorCode}});
+	}
+
+return $response;
 }
 
 =item * $rajce->_close_album($album);
@@ -313,7 +405,7 @@ Close album after adding pictures.
 sub _close_album {
 	my ($self,$album) = @_;
 
-	my $create = {'request'=>{
+	my $request = {'request'=>{
 			'command'=>['closeAlbum'],
 			'parameters'=>{
 				'token'=>[$self->{sessionToken}],
@@ -322,10 +414,18 @@ sub _close_album {
 		}
 	};
 
-	my $open = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($create, KeepRoot => 1, XMLDecl => $self->{XML})});
+	my $xml = XMLout($request, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $close = $self->{BOT}->post($self->{API}, {'data' => $xml});
+	$self->_debug($close->content());
 
-return XMLin($open->content());
+	my $response = XMLin($close->content());
+
+	if($response->{errorCode}){
+		confess($self->{ERRORS}{$response->{errorCode}});
+	}
+
+return $response;
 }
 
 =item * $rajce->add_photo($filename,$album);
@@ -351,7 +451,7 @@ sub add_photo {
 
 	my ($width, $height) = $pic->Get('width','height');
 	
-	my $newpicture = {'request'=>{
+	my $request = {'request'=>{
 			'command'=>['addPhoto'],
 			'parameters'=>{
 				'token'=>[$self->{sessionToken}],
@@ -363,14 +463,28 @@ sub add_photo {
 	};
 
 	$self->_open_album($album);
-	my $obrazek = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($newpicture, KeepRoot => 1,	XMLDecl => $self->{XML}),
+
+	my $xml = XMLout($request, KeepRoot => 1,	XMLDecl => $self->{XML});
+	$self->_debug($xml);
+
+	my $picture = $self->{BOT}->post($self->{API},
+		{'data' => $xml,
 			'thumb' => [undef,$filename,Content => $thumb->ImageToBlob()],
 			'photo' => [undef,$filename,Content => $pic->ImageToBlob()]},
 		Content_Type => 'form-data');
+
+
+	$self->_debug($picture->content());
+	my $response = XMLin($picture->content());
+
+
+	if($response->{errorCode}){
+		confess($self->{ERRORS}{$response->{errorCode}});
+	}
+
 	$self->_close_album($album);
 
-return XMLin($obrazek->content());
+return $response;
 }
 
 =item * $rajce->get_albumurl($album);
@@ -388,10 +502,16 @@ sub get_albumurl {
 		}
 	};
 
-	my $alb = $self->{BOT}->post($self->{API},
-		{'data' => XMLout($url, KeepRoot => 1, XMLDecl => $self->{XML})});
+	my $xml = XMLout($url, KeepRoot => 1, XMLDecl => $self->{XML});
+	$self->_debug($xml);
+	my $alb = $self->{BOT}->post($self->{API}, {'data' => $xml});
+	$self->_debug($alb->content());
 	my $response = XMLin($alb->content());
 
+	if($response->{errorCode}){
+		confess($self->{ERRORS}{$response->{errorCode}});
+	}
+
 return $response->{url};
 }
 
@@ -428,7 +548,6 @@ by the Free Software Foundation; or the Artistic License.
 
 See http://dev.perl.org/licenses/ for more information.
 
-
 =cut
 
 =head1 SEE ALSO
